from unv2x import *

if __name__=='__main__':
    helpmsg=""" UNV2Abaqus: Convert UNV file from Salome to Abaqus INP mesh
        usage: UNV2Abaqus unvfile prefix
        what it does: read unvfile, create an internal FEM object structure
                      in memory and writes the prefix.inp file
        All elements in groups named X_* will just be deleted before conversion
        (usefull to delete all additionnal 1D & 2D elements created by Salome)
        For the moment, all Beam elements are automatically deleted...
        & only linear tri, quads, tetra, hexa & quadratic tri, quads, tetra & hexa
        are supported
    """
    if len(sys.argv)==3:
        unvfile=sys.argv[1]
        prefix=sys.argv[2]
        ls=os.linesep
        # read UNV file in FEM object structure
        UNV=UNVParser(unvfile)
        FEM=UNV.parse()
        # write ELMER UNV file
        # units
        fil=open(prefix + '.inp','w')
        fil.write('*HEADER'+ls)
        fil.write('Created by UNV2Abaqus.py,J.Cugnoni,www.caelinux.com,2006' + ls)
        # node         
        fil.write('*NODE, NSET=ALLNODES, SYSTEM=R'+ls)
        for node in FEM.nodes:
            fil.write(('%d,%10e,%10e,%10e'+ls) % (node.id,node.coords[0],node.coords[1],node.coords[2]))
        # elements
        #  first delete all elems from X_* groups
        X_Ids=[]        
        for group in FEM.elemsets:
            if group.name.startswith('X_'):
                X_Ids.extend(group.items)
                FEM.elemsets.remove(group)                
        FEM.nelemsets=len(FEM.elemsets)        
        #  split element list by types
        types={41:'STRI35',42:'STRI65',44:'S4R5',45:'S8R5',111:'C3D4',115:'C3D8',116:'C3D20',118:'C3D10'}
        elemdic={41:[],42:[],44:[],45:[],111:[],115:[],116:[],118:[]}
        for elem in FEM.elems:
            if elem.type in types.keys():
                elemdic[elem.type].append(elem)
        # write elements
        for typ in elemdic.keys():
            if len(elemdic[typ])>0:                
                fil.write('*ELEMENT,TYPE='+types[typ]+',ELSET='+types[typ]+ls)
                for elem in elemdic[typ]:
                    if not(elem.id in X_Ids):
                        nnodes=elem.nnodes
                        fil.write('%8d,' % elem.id )
                        count=0
                        lst=elem.cntvt
                        for i in range(nnodes):
                            count=count+1
                            if (count<8)&(i<>nnodes-1):
                                fil.write('%8d,' % lst.pop(0))
                            else:
                                fil.write(('%8d'+ls) % lst.pop(0))
                                count=0
        # group definitions
        for group in FEM.nodesets:
            fil.write('*NSET,NAME=%s' % group.name)
            count=0            
            lst=group.items                        
            for i in range(group.nitems):
                count=count+1
                if (count<4)&(i<>group.nitems-1):
                    fil.write('%8d,' % (lst.pop(0)))
                else:
                    fil.write(('%8d'+ls) % (lst.pop(0)))
                    count=0
        for group in FEM.elemsets:
            fil.write('*ELSET,NAME=%s' % group.name)
            count=0            
            lst=group.items                        
            for i in range(group.nitems):
                count=count+1
                if (count<4)&(i<>group.nitems-1):
                    fil.write('%8d,' % (lst.pop(0)))
                else:
                    fil.write(('%8d'+ls) % (lst.pop(0)))
                    count=0
        fil.close()
        
    else:
        print(helpmsg)
