from unv2x import *

if __name__=='__main__':
    helpmsg=""" UNV2ELMER: Convert UNV file from Salome to ELMER mesh
        usage: UNV2X unvfile prefix
        what it does: read unvfile, create an internal FEM object structure
                      in memory and writes the following files:
                      prefix.nodes, prefix.elems, prefix.boundary, prefix.header
        As Salome does not write a color for finite elements, the different bodies
        cannot be distinguished. To define bodies, you need to create element groups
        called body1, body2 etc.. in Salome before you export the mesh.
        Moreover all elements in groups named X_* will just be deleted before conversion
        (usefull to delete all additionnal 1D & 2D elements created by Salome)
    """
    if len(sys.argv)==3:
        unvfile=sys.argv[1]
        prefix=sys.argv[2]
        # read UNV file in FEM object structure
        UNV=UNVParser(unvfile)
        FEM=UNV.parse()
        # write files
        ls=os.linesep
        # node file
        nf=open(prefix + '.nodes','w')
        for node in FEM.nodes:
            nf.write(('%6d 0 %10g %10g %10g'+ls) % (node.id,node.coords[0],node.coords[1],node.coords[2]))
        nf.close()
        # element file
        #  first delete all elems from X_* groups
        X_Ids=[]
        for group in FEM.elemsets:
            if group.name.startswith('X_'):
                X_Ids.extend(group.items)
                FEM.elemsets.remove(group)
        FEM.nelemsets=len(FEM.elemsets)
        for elem in FEM.elems:
            if elem.id in X_Ids:
                FEM.elems.remove(elem)
        FEM.nelems=len(FEM.elems)
        #  set colors corresponding to bodies
        bodylst=[]
        for group in FEM.elemsets:
            if group.name.startswith('body'):
                bodyId=int(group.name.strip()[4:])
                lst=group.items
                bodylst.append(lst)
        for elem in FEM.elems:
            for i in range(len(bodylst)):
                if elem.id in bodylst[i]:
                    idx=FEM.elems.index(elem)
                    FEM.elems(idx).color=i
        ef=open(prefix + '.elems','w')
        for elem in FEM.elems:
            dat=[elem.id,elem.type,elem.material,elem.color,elem.nnodes]
            dat.extend([x for x in elem.cntvt])
            format='%5d, ' * (4 + elem.nnodes) + ' %5d' + ls            
            ef.write(format % tuple(dat))
        ef.close()
        # group file
        grouptypes={7:'NodeSet',8:'ElementSet'}
        gf=open(prefix + '.groups','w')
        for group in (FEM.nodesets + FEM.elemsets):            
            gf.write(('%s(%d) Id: %d'+ls) % (grouptypes[group.type],group.type,group.id))
            gf.write('Name: %s' % (group.name))
            count=0            
            lst=group.items                        
            for i in range(group.nitems):
                count=count+1
                if (count<8)&(i<>group.nitems-1):
                    gf.write('%5d, ' % lst.pop(0))
                else:
                    gf.write(('%5d'+ls) % lst.pop(0))
                    count=0
        gf.close()
        
    else:
        print(helpmsg)
